@@ -6,76 +6,72 @@ from django.contrib.contenttypes.models import ContentType
 from actstream.exceptions import check_actionable_model
 
 
-def follow(user, actor, send_action=True):
+def follow(user, obj, send_action=True, actor_only=False):
     """
-    Creates a ``User`` -> ``Actor`` follow relationship such that the actor's
-    activities appear in the user's stream.
-    Also sends the ``<user> started following <actor>`` action signal.
+    Creates a relationship allowing the object's activities to appear in the
+    user's stream.
+
     Returns the created ``Follow`` instance.
-    If ``send_action`` is false, no "started following" signal will be created
 
-    Syntax::
+    If ``send_action`` is ``True`` (the default) then a
+    ``<user> started following <object>`` action signal is sent.
 
-        follow(<user>, <actor>)
+    If ``actor_only`` is ``True`` (the default) then only actions where the
+    object is the actor will appear in the user's activity stream. Set to
+    ``False`` to also include actions where this object is the action_object or
+    the target.
 
     Example::
 
-        follow(request.user, group)
-
+        follow(request.user, group, actor_only=False)
     """
     from actstream.models import Follow, action
 
-    check_actionable_model(actor)
+    check_actionable_model(obj)
     follow, created = Follow.objects.get_or_create(user=user,
-        object_id=actor.pk,
-        content_type=ContentType.objects.get_for_model(actor))
+        object_id=obj.pk,
+        content_type=ContentType.objects.get_for_model(obj),
+        actor_only=actor_only)
     if send_action and created:
-        action.send(user, verb=_('started following'), target=actor)
+        action.send(user, verb=_('started following'), target=obj)
     return follow
 
 
-def unfollow(user, actor, send_action=False):
+def unfollow(user, obj, send_action=False):
     """
-    Removes ``User`` -> ``Actor`` follow relationship.
-    Optionally sends the ``<user> stopped following <actor>`` action signal.
+    Removes a "follow" relationship.
 
-    Syntax::
-
-        unfollow(<user>, <actor>)
+    Set ``send_action`` to ``True`` (``False is default) to also send a
+    ``<user> stopped following <object>`` action signal.
 
     Example::
 
         unfollow(request.user, other_user)
-
     """
     from actstream.models import Follow, action
 
-    check_actionable_model(actor)
-    Follow.objects.filter(user=user, object_id=actor.pk,
-        content_type=ContentType.objects.get_for_model(actor)).delete()
+    check_actionable_model(obj)
+    Follow.objects.filter(user=user, object_id=obj.pk,
+        content_type=ContentType.objects.get_for_model(obj)).delete()
     if send_action:
-        action.send(user, verb=_('stopped following'), target=actor)
+        action.send(user, verb=_('stopped following'), target=obj)
 
 
-def is_following(user, actor):
+def is_following(user, obj):
     """
-    Checks if a ``User`` -> ``Actor`` relationship exists.
+    Checks if a "follow" relationship exists.
+
     Returns True if exists, False otherwise.
 
-    Syntax::
-
-        is_following(<user>, <actor>)
-
     Example::
 
         is_following(request.user, group)
-
     """
     from actstream.models import Follow
 
-    check_actionable_model(actor)
-    return bool(Follow.objects.filter(user=user, object_id=actor.pk,
-        content_type=ContentType.objects.get_for_model(actor)).count())
+    check_actionable_model(obj)
+    return bool(Follow.objects.filter(user=user, object_id=obj.pk,
+        content_type=ContentType.objects.get_for_model(obj)).count())
 
 
 def action_handler(verb, **kwargs):
@@ -104,4 +100,4 @@ def action_handler(verb, **kwargs):
             setattr(newaction, '%s_content_type' % opt,
                     ContentType.objects.get_for_model(obj))
 
-    newaction.save()
\ No newline at end of file
+    newaction.save()
